/*
** Module   : GenericWebServer.java
** Abstract : generic design for an embedded web server.
**
** Copyright (c) 2009-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131105 First version based on jetty 9.1
*/

package com.goldencode.p2j.web;

import java.net.*;
import java.util.*;
import java.util.logging.*;

import javax.net.ssl.*;

import org.eclipse.jetty.http.*;
import org.eclipse.jetty.server.*;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.*;
import org.eclipse.jetty.util.ssl.*;
import org.eclipse.jetty.util.thread.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.security.SecurityManager;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.web.test.*;

/**
 * A generic design for an embedded web server
 * having support for WebSocket's.
 * 
 * @author mag
 * 
 */
public class GenericWebServer
{

   /** Logger. */
   private static final Logger LOG = 
            LogHelper.getLogger(MainPageHandler.class.getName());

   /** The bootstrap configuration */
   private BootstrapConfig config;

   /** Embedded jetty server */
   private Server server;

   /** Context Handlers */
   private HandlerCollection handlerCollection = new HandlerCollection();

   /** Context path */
   private String contextPath = "/";

   /** Server port */
   private int port;

   /** Server host name */
   private String host;

   /**
    * Startup the embedded server. 
    * The sever will listen on all interfaces. 
    * The port number is dynamically allocated by the OS.
    * 
    * @throws  Exception
    *          If an error condition occur during the server startup.
    */
   public void startup() 
   throws Exception
   {
      this.startup(null, 0);
   }

   /**
    * Startup server.
    * 
    * @param   host
    *          Host name or IP address. If <b>null</b> server will listen on
    *          all interfaces.
    * @param   port
    *          Port number to listen. If <b>0</b> the port is OS dynamically
    *          allocated.
    * 
    * @throws  Exception
    *          If an error condition occur during the server startup.
    */
   public void startup(String host, int port) 
   throws Exception
   {
      // SSL Context Factory for HTTPS
      SslContextFactory sslContextFactory = getSslContextFactory(config);

      QueuedThreadPool pool = new QueuedThreadPool();
      pool.setDaemon(true);

      server = new Server(pool);
      server.manage(pool);
      server.addConnector(configSslConnector(server, 
                                             host, 
                                             port,
                                             sslContextFactory));
      server.setHandler(configWsContext());
      server.setStopAtShutdown(true);
      server.start();

      // get host and port after server is starting
      getRuntimeParams();

      LOG.logp(Level.INFO, 
               "GenericWebServer.startup()", 
               "", 
               LogHelper.generate("Server URL: https://%s:%d%s", 
                                  getHost(), 
                                  getPort(),
                                  contextPath));
   }

   /**
    * Shutdown server.
    * 
    * @throws  Exception
    *          If the server cannot be stopped.
    */
   public void shutdown() 
   throws Exception
   {
      if (server != null && server.isRunning())
      {
         server.stop();
      }
   }

   /**
    * Waits for server thread pool to die.
    * 
    * @throws  Exception
    *          If any error condition occur.
    */
   public void join() 
   throws Exception
   {
      if (server != null && server.isRunning())
      {
         server.join();
      }
   }

   /**
    * Set the bootstrap configuration structure if any.
    * 
    * @param   config
    *          The <code>BootstrapConfig</code> instance or <b>null</b>.
    */
   public void setBootstrapConfig(BootstrapConfig config)
   {
      this.config = config;
   }

   /**
    * Get the server runtime host and port number.
    * 
    */
   private void getRuntimeParams()
   {
      if (server.isRunning())
      {
         Connector[] connectors = server.getConnectors();
         ServerConnector connector = (ServerConnector) connectors[0];
         port = connector.getLocalPort();
         host = connector.getHost();
      }
   }

   /**
    * Register a list of handlers.
    * 
    * @param   handlers
    *          A list of handlers to be registered.
    */
   public void addHandler(Handler[] handlers)
   {
      for (Handler handler : handlers)
      {
         addHandler(handler);
      }
   }

   /**
    * Register a new handler.
    * 
    * @param   handler
    *          A Handler instance.
    */
   public void addHandler(Handler handler)
   {
      if (handler != null)
      {
         handlerCollection.addHandler(handler);
      }
   }

   /**
    * Register a new WebSocket.
    * 
    * @param   target
    *          The WebSocket target.
    * @param   webSocket
    *          A POJO decorated with <code>@WebSocket</code> annotation.
    */
   public void addWebSocketHandler(String target, Class<?> webSocket)
   {
      Handler wsHandler = new PojoWebSocketHandler(target, webSocket);
      handlerCollection.addHandler(wsHandler);
   }

   /**
    * Construct a SSL Context Factory for HTTPS.
    * 
    * @param   config
    *          The client bootstrap configuration if any. 
    *          When <b>null</b> use default SSL context.
    * @return  The <code>SslContextFactory</code> instance.
    * 
    * @throws  Exception
    *          If any error conditions occur.
    */
   private SslContextFactory getSslContextFactory(BootstrapConfig config)
   throws Exception
   {
      SecurityManager sm = SecurityManager.getInstance();
      SslContextFactory sslContextFactory = new SslContextFactory();
      SSLContext sslContext = (config == null) ? 
         sm.getSecureSocketContext() : 
         sm.getSecureSocketContext(config);
      sslContextFactory.setSslContext(sslContext);

      return sslContextFactory;
   }

   /**
    * Create and configure a SSL server connector.
    * 
    * @param   server
    *          The server instance.
    * @param   host
    *          The server host name or IP address.
    * @param   port
    *          The port number to listen.
    * @param   sslContextFactory
    *          The <code>SslContextFactory</code> instance.
    * 
    * @return  The SSL server connector
    */
   private ServerConnector configSslConnector(Server server, 
                                              String host,
                                              int port, 
                                              SslContextFactory sslContextFactory)
   {
      String protocol = HttpVersion.HTTP_1_1.asString();
      ServerConnector sslConnector = new ServerConnector(server,
         new SslConnectionFactory(sslContextFactory, protocol),
         new HttpConnectionFactory(new HttpConfiguration()));
      sslConnector.setHost(host);
      sslConnector.setPort(port);

      return sslConnector;
   }

   /**
    * Create and configure a server ContextHandler.
    * 
    * @return  The ContextHandler used as server Handler.
    */
   private Handler configWsContext()
   {
      ContextHandler context = new ContextHandler();
      context.setContextPath(contextPath);
      context.setResourceBase(".");
      context.setClassLoader(Thread.currentThread().getContextClassLoader());
      context.setHandler(handlerCollection);

      return context;
   }

   /**
    * Get host site local address.
    * 
    * @return  Site local host address if any or <b>localhost</b> if none.
    */
   private String getHostLocalAddress()
   {
      String hostAddress = "localhost";
      try
      {
         InetAddress address = getSiteLocalAddress();
         if (address != null)
         {
            hostAddress = address.getHostAddress();
         }
      }
      catch (Exception e)
      {
         LOG.logp(Level.SEVERE,
                  "GenericWebServer.getHostLocalAddress()",
                  "",
                  LogHelper.generate("Error!"),
                  e);            
      }
      return hostAddress;
   }

   /**
    * Search on the network interfaces of this machine for
    * <code>InetAddress</code> which is a site local address.
    * 
    * @return  The InetAddress of the first site local address.
    * 
    * @throws  Exception
    *          If an error condition occur.
    */
   private InetAddress getSiteLocalAddress() 
   throws Exception
   {
      // enumerate network interfaces
      Enumeration<NetworkInterface> netInterfaces =
         NetworkInterface.getNetworkInterfaces();
      if (netInterfaces != null)
      {
         while (netInterfaces.hasMoreElements())
         {
            NetworkInterface netInterface = netInterfaces.nextElement();
            if (netInterface.isUp())
            {
               // enumerate InetAddresses bound to this network interface.
               Enumeration<InetAddress> addresses = 
                  netInterface.getInetAddresses();
               while (addresses.hasMoreElements())
               {
                  InetAddress address = addresses.nextElement();
                  if (address.isSiteLocalAddress())
                  {
                     return address;
                  }
               }
            }
         }
      }
      return null;
   }

   /**
    * Set the server context path root.
    * 
    * @param   contextPath
    *          A string representing the context path root.
    */
   public void setContextPath(String contextPath)
   {
      this.contextPath = contextPath;
   }

   /**
    * Get server runtime port number.
    * 
    * @return  The server runtime port number.
    */
   public int getPort()
   {
      return port;
   }

   /**
    * Get the server runtime host.
    * 
    * @return  The server runtime host.
    * 
    */
   public String getHost()
   {
      return (host == null) ? getHostLocalAddress() : host;
   }

}
