/*
** Module   : PojoWebSocketHandler.java
** Abstract : create and register WebSockets POJO objects.
**
** Copyright (c) 2009-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131105 First version based on jetty 9.1
*/

package com.goldencode.p2j.web;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.eclipse.jetty.server.*;
import org.eclipse.jetty.websocket.server.*;
import org.eclipse.jetty.websocket.servlet.*;

/**
 * This class is used to register and handle WebSockets POJO objects. 
 * <p>
 * Methods to create WebSockets:
 * <ul>
 *    <li>POJO WebSocket using annotations.</li>
 *    <li>Implements WebSocketListener interface.</li>
 *    <li>Extends WebSocketAdapter class.</li>
 * </ul> * 
 */
public class PojoWebSocketHandler 
extends WebSocketHandler
{
   /** WebSocket class */
   private Class<?> webSocketPojo;
   
   /** WebSocket target should be unique on the server context */ 
   private String webSocketTarget;

   /**
    * Constructor.
    * 
    * @param   webSocketTarget
    *          The WebSocket target string.
    *          This value should be unique on the WebSocket collection
    *          of handlers for the same server context.
    * @param   webSocketPojo
    *          The WebSocket POJO class.
    */
   public PojoWebSocketHandler(String webSocketTarget, Class<?> webSocketPojo)
   {
      this.webSocketTarget = webSocketTarget;
      this.webSocketPojo = webSocketPojo;
   }

   /**
    * Register a WebSocket class POJO with the 
    * default {@link WebSocketCreator}.
    *
    * @param  wsServletFactory
    *         Basic WebSocketServletFactory for working with 
    *         Jetty-based WebSocketServlets
    */
   @Override
   public void configure(WebSocketServletFactory wsServletFactory)
   {
      wsServletFactory.register(webSocketPojo);
   }

   /**
    * Handle WebSocket's upgrade requests.
    * If the request is not handled and the target is equals to
    * this target handler the set handled to <code>true</code>
    *
    * @param  target 
    *         The target of the request - either a URI or a name.
    * @param  baseRequest 
    *         The original unwrapped request object.
    * @param  request 
    *         The request either as the {@link Request}
    *         object or a wrapper of that request. 
    *         The {@link HttpChannel#getCurrentHttpChannel()}
    *         method can be used to access the Request object if required.
    * @param  response The response as the {@link Response}
    *         object or a wrapper of that request. 
    *         The {@link HttpChannel#getCurrentHttpChannel()}
    *         method can be used access the Response object if required.
    *
    * @throws IOException
    * @throws ServletException
    */
   @Override
   public void handle(String target, 
                      Request baseRequest,
                      HttpServletRequest request, 
                      HttpServletResponse response)
   throws IOException, 
          ServletException
   {
      if (!baseRequest.isHandled() && webSocketTarget.equals(target))
      {
         super.handle(target, baseRequest, request, response);
         baseRequest.setHandled(true);
      }
   }

}
